##
#
# YAVLI RESILIENCE API
#
# This is a Ruby implementation of our Yavli Resilience API.
# It provides functionality to retrieve our AdServer code from our
# server, caching it to a file, and to read out and return the
# AdServer code so it can be printed to the pages of your website.
#
##

require "singleton"
require "logger"
require "json"
require "uri"
require "net/http"
require "openssl"
require_relative "YResilience-Config"

class YavliResilienceAPI

	# The API is a Singleton class - get an instance by YavliResilienceAPI.instance
	include Singleton

	# Get the Yavli Resilience Config
	include YavliResilienceConfig

	def initialize

		# Logging level is configured in YResilience-Config.rb
		@log = Logger.new(LOG_TARGET)

		# Holds AdServer code after being read from the cache file
		@data = nil

		# Holds status messages when updating the AdServer code
		@apiStatus = nil
	end

	# Returns the AdServer Code that you should print out into your web pages
	def getAdServerCode

		# If @data is empty, call loadData to read the cache file
		if @data.nil? || @data.empty? then
			loadData
		end

		# If @data is now populated, return its contents
		if !@data.nil? && !@data.empty? then
			return @data
		end

		@log.error("Couldn't retrieve AdServer code from file.")

		return nil
	end

	# Reads the cache file and outputs its contents to @data
	# CACHE_FILE set in YResilience-Config.rb
	def loadData
		begin
			if File.exist?(CACHE_FILE) then
				fileData = File.read(CACHE_FILE)
				if !fileData.nil? && !fileData.empty? then
					dataJSON = JSON.parse fileData
					if !dataJSON.nil? && !dataJSON.empty? && !dataJSON['data'].nil? && !dataJSON['data'].empty? && !dataJSON['data']['init_js'].nil? && !dataJSON['data']['init_js'].empty? then

						# Populate @data only if the cache file has the proper structure and contents
						@data = dataJSON["data"]["init_js"]
					end
				end
			end
		rescue StandardError => e

			# Graceful bailout on any file or parsing errors
			@log.error("Could parse AdServer code from JSON file: " + e.message)
		end
	end

	# Pulls updated AdServer code, saves it into the cache file and repopulates @data
	def refreshAdServerCode

		# Get the new AdServer code from Yavli Server. Returns false on error
		dataJSON = apiRequest("/get-ad-server")

		# Try to parse and write the new AdServer code to the cache file and repopulate @data
		if (dataJSON != false) then

			# Check JSON structure
			if !dataJSON.nil? && !dataJSON.empty? && !dataJSON['data'].nil? && !dataJSON['data'].empty? && !dataJSON['data']['init_js'].nil? && !dataJSON['data']['init_js'].empty? then
				begin

					# Convert JSON object to string and try to replace the contents of the cache file
					dataString = dataJSON.to_json
					File.open(CACHE_FILE, "w") do |file|
						file.truncate(0)
						file.write(dataString)
					end

					# Populate @data only if the cache file has the proper structure and contents 
					@data = dataJSON["data"]["init_js"]

					@apiStatus = "Successfully updated AdServer Code"
					return true
				rescue StandardError => e

					# Graceful bailout on any file or parsing errors
					@apiStatus = "Could not save JSON to file: " + e.message
				end
			else
				@apiStatus = "Invalid JSON structure from YAVLI API"
			end
		end

		return false
	end

	# Makes a call to Yavli Server to retrieve AdServer code
	def apiRequest(action)

		# Construct the full request URL for the call.
		# API_URL set in YResilience-Config.rb
		requestURL = API_URL + action

		# Check if credentials are set in YResilience-Config.rb
		if WEBSITE_ID.nil? || WEBSITE_ID.empty? || API_KEY.nil? || API_KEY.empty? then
			@apiStatus = "Credentials not configured. Could not update AdServer Code"
			return false
		end

		# Initialize HTTP request
		apiURI = URI.parse(requestURL)
		httpInst = Net::HTTP.new(apiURI.host, apiURI.port)
		
		if apiURI.scheme == "https" then 
			httpInst.use_ssl = true
			httpInst.verify_mode = OpenSSL::SSL::VERIFY_NONE
		end
		
		httpReq = Net::HTTP::Get.new(apiURI.request_uri)
		httpReq.initialize_http_header({"X-WebsiteID" => WEBSITE_ID, "X-ApiKey" => API_KEY})

		# Try to execute HTTP call to Yavli server and verify result
		begin
			httpResp = httpInst.request(httpReq)
		rescue StandardError => e
			@apiStatus = "Could not connect to YAVLI API: " + e.message
			return false
		end

		# HTTP Response object check
		if httpResp.nil? || httpResp.code.nil? || httpResp.code.empty? then
			@apiStatus = "Unsuccessful request to YAVLI API"
			return false
		end

		# HTTP Response code check
		if httpResp.code != "200" then
			@apiStatus = "Unsuccessful request to YAVLI API: Code " + httpResp.code + ", Body: " + httpResp.body
			return false
		end

		# HTTP Response body check
		if httpResp.body.nil? || httpResp.body.empty? then
			@apiStatus = "Unsuccessful request to YAVLI API: Empty body"
			return false
		end

		# Try to parse JSON in the HTTP Response body
		begin
			dataJSON = JSON.parse httpResp.body
		rescue StandardError => e
			@apiStatus = "Could not parse JSON: " + e.message
			return false
		end

		# JSON content check
		if dataJSON.nil? || dataJSON.empty? then
			@apiStatus = "Cannot parse JSON sent from YAVLI API"
			return false
		end

		# Check if the JSON itself contains a Yavli API error
		if dataJSON.has_key?("error") then
			@apiStatus = "YAVLI API returned error " + dataJSON["error"]["code"] + ": " + dataJSON["error"]["status"] + ", " + dataJSON["error"]["detail"]
			return false
		end

		# Return valid JSON
		return dataJSON
	end

	# Return any API status messages
	def getApiStatus
		return @apiStatus
	end
end